package commands

import (
	"archive/tar"
	"bytes"
	"encoding/hex"
	"errors"
	"fmt"
	"io"
	"os"

	"git.cheetah.cat/cheetah/moto-flash-data/flashpart"
	"github.com/rs/zerolog/log"
)

type MparPaxListCommand struct {
	SourceFileName string `long:"file" short:"f" required:"true" description:"input-filename"`
}

func (command *MparPaxListCommand) Execute(args []string) error {
	inputFile, err := os.Open(command.SourceFileName)
	if err != nil {
		return err
	}
	defer inputFile.Close()

	inputFileStat, err := inputFile.Stat()
	if err != nil {
		return err
	}

	log.Info().Int64("fileSize", inputFileStat.Size()).Msg("Reading raw file...")
	pdata := make([]byte, inputFileStat.Size())
	_, err = io.ReadFull(inputFile, pdata)
	if err != nil {
		return err
	}

	fpHeader := flashpart.ParseFlashPartHeader(pdata[:32])
	log.Info().Msg(
		fmt.Sprintf("Media-ID: 0x%02X, Partition Size: 0x%08X / %d bytes, Partition Tag: %s",
			fpHeader.MediaID,
			fpHeader.TotalSize,
			fpHeader.TotalSize,
			fpHeader.VersionText,
		),
	)
	log.Info().Msg(
		fmt.Sprintf("Original-Full-Header-Hex: %s", hex.EncodeToString(pdata[:32])),
	)

	if len(pdata[32:]) != len(pdata)-32 {
		return errors.New("something doesnt add up")
	}
	if len(pdata[32:]) != int(fpHeader.TotalSize)-32 {
		log.Error().Uint32("header ts", fpHeader.TotalSize).Uint32("len pdata", uint32(len(pdata))).Msg("size mismatch")
		return nil
	}

	breader := bytes.NewReader(pdata[32:])
	tarReader := tar.NewReader(breader)

	for {
		header, err := tarReader.Next()
		if err == io.EOF {
			break
		}

		if err != nil {
			log.Error().Err(err)
		}
		fmt.Println(header)

		// if header.Typeflag == tar.TypeReg && !strings.Contains(header.Name, "._") {
		// 	// content, err := io.ReadAll(tarReader)
		// 	// if err != nil {
		// 	// 	log.Error().Err(err)
		// 	// }

		// 	fmt.Println("FILE_NAME: ", header.Name)
		// 	///fmt.Println(string(content))
		// }
	}
	return nil
}
